<?php 
/**
* @author  		Michal Baniowski <baniczek@gmail.com>
* @link  		http://www.michalbaniowski.pl
* 
* @package 		bform
*/
class Bform_Core_Driver_Collection {

    /**
     * Drivers container
     * 
     * @var             array Array of drivers
     * @access          protected
     */
    protected $_elements = array();

    public function __set($name, Bform_Driver $value)
    {
        if (isset($this->_elements[$name]))
        {
            throw new Bform_Exception('Driver :name exists!', array(':name' => $name));
        }
        $this->_elements[$name] = $value;
    }

    public function & __get($name) 
    {
        $null = NULL;
        
        if (isset($this->_elements[$name])) 
        {
            return $this->_elements[$name];
        } 
        else 
        {
            return $null;
        }
    }
    
    public function __isset($name)
    {
        return isset($this->_elements[$name]);
    }

    /**
     * Returns all form drivers
     * 
     * @param           integer $type
     * @return          array
     */
    public function get_all($type = NULL)
    {
        if ($type === NULL)
        {
            $type = Bform_Form::DRIVERS_NOT_BUTTONS & ~ Bform_Form::DRIVERS_ONLY_BUTTONS;
        }
        
        $_get_only_buttons = ($type & Bform_Form::DRIVERS_ONLY_BUTTONS) ? TRUE : FALSE;
        $_get_not_buttons = ($type & Bform_Form::DRIVERS_NOT_BUTTONS) ? TRUE : FALSE;
        
        $result = array();
        foreach ($this->_elements as $e)
        {
            if ($_get_only_buttons AND ! ($e instanceof Bform_Driver_Button))
            {
                continue;
            }
            if ($_get_not_buttons AND $e instanceof Bform_Driver_Button)
            {
                continue;
            }
            $result[] = $e;
        }
        return $result;
    }
}