<?php 
/**
* @author  		Michal Baniowski <baniczek@gmail.com>
* @link  		http://www.michalbaniowski.pl
* 
* @package 		bform
*/
class Bform_Core_Driver_Group extends Bform_Driver {

    public $elements = array();
    
    public static function factory(Bform_Form $form, $name, array $elements = array(), array $info = array())
    {
        return new Bform_Driver_Group($form, $name, $elements, $info);
    }
    
    protected function _set_value($value)
    {
        if ($this->is_array) 
        {
            $form = $this->form;
            $path = $this->path;

            if ($form->is_sent() AND Arr::path($form->data(), $path, NULL, '.') === NULL)
            {
                return array();
            }
            foreach ($this->elements as $e) 
            {
                if (in_array($e->value, $value)) 
                {
                    $e->checked = TRUE;
                }
                else 
                {
                    $e->checked = FALSE;
                }
            }
        } 
        else
        {
            foreach ($this->elements as $e) 
            {
                if ($e->value == $value) 
                {
                    $e->checked = TRUE;
                }
                else 
                {
                    $e->checked = FALSE;
                }
            }
        }
        
        return $value;
    }
    
    protected function _get_value($value)
    {
        if ($this->is_array) 
        {
            $values = array();
            foreach ($this->elements as $e) 
            {
                if ($e->checked) 
                {
                    $values[] = $e->value;
                }
            }
            return $values;
        }
        else
        {
            foreach ($this->elements as $e) 
            {
                if ($e->checked) 
                {
                    return $e->value;
                }
            }
        }
    }

    public function __construct(Bform_Form $form, $name, array $elements, array $info = array())
    {
        parent::__construct($form, $name, $info);

        if ($this->is_array)
        {
            foreach ($elements as $value => $label)
            {
                if (is_object($label))
                {
                    if ( ! ($label instanceof Bform_Driver_Group_Checkbox))
                    {
                        throw new Bform_Exception("Object must be instance of Bform_Driver_Group_Checkbox!");
                    }
                    $this->elements[] = $label;
                }
                else
                {
                    $this->elements[] = Bform_Driver_Group_Checkbox::factory($this->form, $this->name, array('label' => $label, 'value' => $value, 'required' => $this->required, 'class' => $this->class, 'parent' => $this));
                }
            }
        }
        else
        {
            foreach ($elements as $value => $label)
            {
                if (is_object($label))
                {
                    if ( ! ($label instanceof Bform_Driver_Group_Radio))
                    {
                        throw new Bform_Exception("Object must be instance of Bform_Driver_Group_Radio!");
                    }
                    $this->elements[] = $label;
                }
                else
                {
                    $this->elements[] = Bform_Driver_Group_Radio::factory($this->form, $this->name, array('label' => $label, 'value' => $value, 'required' => $this->required, 'class' => $this->class, 'parent' => $this));
                }
            }
        }
        
        if (count($elements))
        {
            $this->refresh();
        }
    }
    
    /**
     * Refresh group elements
     */
    public function refresh()
    {
        foreach ($this->elements as $e)
        {
            $e->parent = $this;
        }
    }
}