<?php 
/**
* @author  		Michal Baniowski <baniczek@gmail.com>
* @link  		http://www.michalbaniowski.pl
* 
* @package 		bform
*/
abstract class Bform_Core_Plugin {
    
    const RESULT_NONE           = 0x00000001;
    const RESULT_PLUGIN         = 0x00000010;
    const RESULT_FORM           = 0x00000100;
    const RESULT_FORM_VALUE     = 0x00001000;
    const RESULT_DRIVER         = 0x00010000;
    const RESULT_DRIVER_VALUE   = 0x00100000;
    
    /**
     *
     * @var         Bform_Plugin_Manager
     */
    protected $_manager = NULL;
    
    /**
     * Plugin return value.
     * @var         mixed
     */
    protected $_value = NULL;
    
    /**
     *
     * @var         Bform_Driver
     */
    private $_driver = NULL;
        
    public function __construct(Bform_Plugin_Manager $manager)
    {
        $this->_manager = $manager;
    }
    
    /**
     *
     * @return      mixed
     */
    public function value()
    {
        return $this->_value;
    }
    
    /**
     *
     * @return          Bform_Driver
     */
    public function driver()
    {
        return $this->_driver;
    }
    
    /**
     *
     * @return      Bform_Form
     */
    final public function form()
    {
        return $this->_manager->form();
    }
    
    public static function method_name($name, $type)
    {
        if ($type === Bform_Plugin_Manager::METHOD_DRIVER)
        {
            $method_name = "extend_driver_$name";
        }
        elseif ($type === Bform_Plugin_Manager::METHOD_FORM)
        {
            $method_name = "extend_form_$name";
        }
        elseif ($type === Bform_Plugin_Manager::METHOD_PLUGIN)
        {
            $method_name = "plugin_$name";
        }
        
        return $method_name;
    }
    
    public function method_exists($function_name, $type)
    {
        if (method_exists($this, self::method_name($function_name, $type)))
        {
            return TRUE;
        }
        
        return FALSE;
    }
    
    public function update(Bform_Driver $driver)
    {
        return FALSE;
    }
    
    public function execute($function_name, $values, $type, $driver = NULL)
    {
        $method_name = self::method_name($function_name, $type);
        
        $this->_driver = $driver;
        
        $result = call_user_func(array($this, $method_name), $values);
        return $result;
    }
}