<?php 
/**
* @author  		Michal Baniowski <baniczek@gmail.com>
* @link  		http://www.michalbaniowski.pl
* 
* @package 		bform
*/
class Bform_Core_Plugin_Manager {
    
    const METHOD_FORM           = 'form';
    const METHOD_DRIVER         = 'driver';
    const METHOD_PLUGIN         = 'plugin';
    
    /**
     *
     * @var         Bform_Form
     * @access      protected
     */
    protected $_form = NULL;
    
    /**
     *
     * @var         array
     * @access      protected
     */
    protected $_plugins = array();
    
    /**
     *
     * @var         Bform_Plugin
     * @access      protected
     */
    protected $_last_plugin = NULL;
    
    /**
     *
     * @var         mixed
     */
    protected $_last_plugin_value = NULL;
    
    /**
     *
     * @var         bool
     */
    protected $_disabled = FALSE;
    
    public function __construct(Bform_Form $form)
    {
        $this->_form = $form;
        
        $plugins = Kohana::$config->load('bform.plugins.enabled');
        if ( ! is_array($plugins))
        {
            throw new Bform_Exception('Check API!');
        }
        
        foreach ($plugins as $name)
        {
            if ( ! class_exists($name))
            {
                throw new Bform_Exception("Plugin ':name' doesn't exists!", array(':name' => $name));
            }
            $plugin = new $name($this);
            $this->_plugins[strtolower($name)] = $plugin;
        }
    }
    
    public function execute($name, $arguments, $type, $driver = NULL)
    {
        if ($this->_disabled)
        {
            return Bform_Plugin::RESULT_FORM;
        }
        
        if ($type === self::METHOD_DRIVER)
        {
            $result = $this->_execute_driver($name, $arguments, $driver);
        }
        elseif ($type === self::METHOD_FORM)
        {
            $result = $this->_execute_form($name, $arguments);
        }
        
        if ($result === NULL)
        {
            throw new Bform_Plugin_Exception("Your plugin method must return smth!");
        }
        
        if ($result & Bform_Plugin::RESULT_NONE)
        {
            return $result;
        }
        elseif ($result & Bform_Plugin::RESULT_PLUGIN)
        {
            return $result;
        }
        elseif ($result & Bform_Plugin::RESULT_FORM)
        {
            $last_plugin = $this->_last_plugin;
            $this->_last_plugin = NULL;
            
            if ($result & Bform_Plugin::RESULT_FORM_VALUE)
            {
                $this->_last_plugin_value = $last_plugin->value();
            }
                        
            return $result;
        }
        elseif ($result & Bform_Plugin::RESULT_DRIVER)
        {
            $last_plugin = $this->_last_plugin;
            $this->_last_plugin = NULL;
            
            if ($result & Bform_Plugin::RESULT_DRIVER_VALUE)
            {
                $this->_last_plugin_value = $last_plugin->value();
            }
            
            return $result;
        }
        else
        {
            throw new Bform_Exception('Check API!');
        }
    }
    
    private function _execute_driver($name, $arguments, $driver)
    {
        $result = Bform_Plugin::RESULT_NONE;
        
        foreach ($this->_plugins as $p)
        {
            if ($p->method_exists($name, self::METHOD_DRIVER))
            {
                $result = $p->execute($name, $arguments, self::METHOD_DRIVER, $driver);
                $this->_last_plugin = $p;
                break;
            }
        }
        
        return $result;
    }
    
    private function _execute_form($name, $arguments)
    {
        $result = Bform_Plugin::RESULT_NONE;
        
        if (empty($this->_last_plugin))
        {
            foreach ($this->_plugins as $p)
            {
                if ($p->method_exists($name, self::METHOD_FORM))
                {
                    $result = $p->execute($name, $arguments, self::METHOD_FORM);
                    $this->_last_plugin = $p;
                    break;
                }
            }
        }
        else
        {
            if ($this->_last_plugin->method_exists($name, self::METHOD_PLUGIN))
            {
                $result = $this->_last_plugin->execute($name, $arguments, self::METHOD_PLUGIN);
            }
            else
            {
                throw new Bform_Exception('Your plugin method ":name" doesn\'t exists!', array(':name' => $name));
            }
        }
        
        return $result;
    }
    
    public function update(Bform_Driver $driver)
    {
        foreach ($this->_plugins as $p)
        {
            $p->update($driver);
        }
    }
    
    public function value()
    {
        return $this->_last_plugin_value;
    }
    
    public function form()
    {
        return $this->_form;
    }
    
    public function disabled($value = NULL)
    {
        if ($value === NULL)
        {
            return $this->_disabled;
        }
        $this->_disabled = $value;
    }
}