<?php 
/**
* @author  		Michal Baniowski <baniczek@gmail.com>
* @link  		http://www.michalbaniowski.pl
* 
* @package 		bform
*/
abstract class Bform_Core_Validator {
    
    /**
     * An error message what gone wrong
     * 
     * @var             string
     */
    protected $_message = '';
    
    /**
     * A transformed error message
     * @var             string
     */
    private $_message_parsed = '';
    
    /**
     * Additional params passed thru add_validator() method
     * 
     * @see             Bform_Form::add_validator()
     * @var             array
     */
    protected $_options = array();
    
    /**
     * A driver belong to instance of Bform_Validator_*
     * 
     * @var             Bform_Driver 
     * @access          protected
     */
    protected $_driver = NULL;

    public function __construct(array $options = array())
    {
        $this->_options = $options;
        if ( ! empty($this->_options['message']))
        {
            $this->_message = $this->_options['message'];
        }
    }

    /**
     * Updates a value to validate
     * 
     * @access public
     * @see Bform_Form::validate()
     */
    public function update() 
    {
        // todo
    }

    /**
     * Validates a value. This method should be extended in your own validator
     * 
     * @return void
     */
    abstract public function validate();
    
    /**
     * Returns a i18n string error message
     * 
     * @return          string 
     */
    public function message()
    {
        $this->_message_parsed OR $this->_parse_message();
        return $this->_message_parsed;
    }
    
    /**
     * ...
     * 
     * @return          void
     */
    final protected function _parse_message()
    {
        $error = $this->_message;
        $new_vars = array();
        
        foreach ($this->_options as $name => $value) 
        {
            if (is_array($value))
            {
                continue;
            }
            
            if ($name[0] == '_') 
            {
                $name = substr($name, 1);
            }
            
            $name = ':'.$name;
            $new_vars[$name] = $value;
        }
        
        if ($this instanceof Bform_Validator_Driver)
        {
            $new_vars[':value'] = $this->driver()->value;
        }
        
        $error = __($error, $new_vars);
        
        $this->_message_parsed = $error;
    }
}