<?php 
/**
* @author  		Michal Baniowski <baniczek@gmail.com>
* @link  		http://www.michalbaniowski.pl
* 
* @package 		bform
*/
class Bform_Plugin_Blocks extends Bform_Plugin {
    
    const BLOCK_FIELDSET = 'fieldset';
    
    protected $_form_methods = array('block_start', 'block_end', 'blocks_get_to_open', 'blocks_get_to_close');
    
    protected $_current_block = NULL;
    protected $_current_block_view = NULL;
    protected $_current_driver = NULL;
    protected $_blocks = array();
    protected $_blocks_opened = array();
    protected $_blocks_opened_view = array();
    protected $_drivers = array();
    
    public function extend_form_block_start(array $args)
    {
        $block = Bform_Plugin_Blocks_Block::factory($args);
        $block->parents($this->_blocks_opened);
        
        $this->_blocks_opened[] = $block;
        $this->_current_block = $block;
        $this->_blocks[] = $block;
        
        return self::RESULT_FORM;
    }
    
    public function extend_form_block_end()
    {
        array_pop($this->_blocks_opened);
        $this->_current_block = end($this->_blocks_opened);
        return self::RESULT_FORM;
    }
    
    public function extend_form_blocks_get_to_open(array $args)
    {
        if (empty($args[0]) OR ! ($args[0] instanceof Bform_Driver))
        {
            throw new Bform_Exception('check api!');
        }
        $driver = $args[0];
        
        $result = FALSE;
        foreach ($this->_blocks as $block)
        {
            $drivers = $block->drivers();
            if (in_array($driver, $drivers, TRUE))
            {
                $result = $block->parents();
                $result[] = $block;
                $copy = $result;
                
                foreach ($result as $key2 => $block2)
                {
                    if (in_array($block2, $this->_blocks_opened, TRUE))
                    {
                        unset($result[$key2]);
                    }
                }
                $this->_blocks_opened = $copy;
                $this->_blocks_opened_view = $result;
                $this->_current_block_view = end($result);
                break;
            }
        }
        
        $this->_value = $result;
        
        return self::RESULT_FORM | self::RESULT_FORM_VALUE;
    }
    
    public function extend_form_blocks_get_to_close(array $args)
    {
        if (empty($args[0]) OR ! ($args[0] instanceof Bform_Driver))
        {
            throw new Bform_Exception('check api!');
        }
        $driver = $args[0];
        
        $current = end($this->_blocks_opened);
        
        if ( ! $current)
        {
            $this->_value = FALSE;
            return self::RESULT_FORM | self::RESULT_FORM_VALUE;
        }
        
        $opened = $current->parents();
        $opened[] = $current;
        $to_close = array();
        $next_driver = $this->next_driver($driver);
        $next_driver_block = $this->driver_block($next_driver);
        
        foreach ($opened as $key => $block)
        {
            if ( ! $next_driver_block)
            {
                $to_close[] = $block;
                continue;
            }
            
            $drivers = $block->drivers();
            $parents = $block->parents();
            $parents_with_self = array_merge($parents, array($block));
            $is_driver_last = ( ! empty($drivers[count($drivers) - 1]) AND ($drivers[count($drivers) - 1] === $driver ? TRUE : FALSE));
            
            if ($is_driver_last)
            {
                foreach ($parents_with_self AS $block2)
                {
                    if ( ! in_array($block2, $next_driver_block->parents(), TRUE) AND ! in_array($next_driver, $block2->drivers(), TRUE))
                    {
                        $to_close[] = $block2;
                    }
                }
            }
        }
        
        $this->_value = $to_close;
        foreach ($to_close as $block)
        {
            array_pop($this->_blocks_opened);
        }
        $this->_current_block_view = end($this->_blocks_opened);
        return self::RESULT_FORM | self::RESULT_FORM_VALUE;
    }
    
    public function update(Bform_Driver $driver)
    {
        if (($block = $this->_current_block))
        {
            $block->driver($driver);
        }
        $this->_drivers[] = $driver;
    }
    
    public function current_block()
    {
        return $this->_current_block;
    }
    
    public function next_driver($driver)
    {
        $result = FALSE;
        
        foreach ($this->_drivers as $key => $d)
        {
            if ($d === $driver AND ! empty($this->_drivers[$key + 1]))
            {
                $result = $this->_drivers[$key + 1];
            }
        }
        
        return $result;
    }
    
    
    public function driver_block($driver)
    {
        foreach ($this->_blocks as $block)
        {
            $drivers = $block->drivers();
            foreach ($drivers as $d)
            {
                if ($d === $driver)
                {
                    return $block;
                }
            }
        }
        return FALSE;
    }
}