<?php 
/**
* @author  		Michal Baniowski <baniczek@gmail.com>
* @link  		http://www.michalbaniowski.pl
* 
* @package 		bform
*/
class Bform_Plugin_Tabs extends Bform_Plugin {
    
    protected $_form_methods = array('add_tab', 'tabs_open', 'tabs_get');
    protected $_driver_methods = array('tabs_is_driver_first', 'tabs_is_driver_last', 'tabs_close');
    
    protected $_tabs = array();
    protected $_current_tab_index = - 1;
    protected $_last_driver_name = '';
    
    public function extend_form_add_tab($args)
    {
        if ( ! count($this->_tabs) AND count($this->form()->drivers->get_all()))
        {
            throw new Bform_Exception('You must create first tab!');
        }
        
        $name = $args[0];
        $info = isset($args[1]) ? $args[1] : array();
        
        if (empty($info['html']['id']))
        {
            $id = 'bform-tab-'.url::title($name);
            $id = strtolower($id);
            $info['html']['id'] = $id;
        }
        if (empty($info['html']['class']))
        {
            $info['html']['class'] = '';
        }
        
        $this->_tabs[] = array('name' => $name, 'info' => $info, 'drivers' => array());
        $this->_current_tab_index = count($this->_tabs) - 1;
        return self::RESULT_FORM;
    }
    
    public function extend_form_tabs_open()
    {
        $this->_value = is_array($this->current_tab());
        return self::RESULT_FORM | self::RESULT_FORM_VALUE;
    }
    
    public function extend_form_tabs_get()
    {
        $this->_value = $this->_tabs;
        return self::RESULT_FORM | self::RESULT_FORM_VALUE;
    }
    
    public function extend_driver_tabs_is_driver_first()
    {
        $tab = $this->_value = FALSE;
        foreach ($this->_tabs as $tab_array)
        {
            if (in_array($this->driver()->name, $tab_array['drivers']) AND array_search($this->driver()->name, $tab_array['drivers']) === 0)
            {
                $tab = $tab_array;
                break;
            }
        }
        $this->_value = $tab;
        return self::RESULT_DRIVER | self::RESULT_DRIVER_VALUE;
    }
    
    public function extend_driver_tabs_is_driver_last()
    {
        $result = FALSE;
        foreach ($this->_tabs as $tab_array)
        {
            if (in_array($this->driver()->name, $tab_array['drivers']) AND array_search($this->driver()->name, $tab_array['drivers']) === count($tab_array['drivers']) - 1)
            {
                $result = TRUE;
                break;
            }
        }
        
        $this->_value = $result;
        return self::RESULT_DRIVER | self::RESULT_DRIVER_VALUE;
    }
    
    public function extend_driver_tabs_close()
    {
        $this->_value = ($this->driver()->name === $this->_last_driver_name AND count($this->_tabs) > 0);
        return self::RESULT_DRIVER | self::RESULT_DRIVER_VALUE;
    }
    
    public function & current_tab()
    {
        if ($this->_current_tab_index === - 1)
        {
            // todo: rence opadajom...
            $false = FALSE;
            $return = & $false;
            return $return;
        }
        
        $tab = & $this->_tabs[$this->_current_tab_index];
        return $tab;
    }
        
    public function update(Bform_Driver $driver)
    {
        if (($current_tab = & $this->current_tab()) !== FALSE)
        {
            $current_tab['drivers'][] = $driver->name;
        }
        $this->_last_driver_name = $driver->name;
    }
}